@@ -3,7 +3,7 @@ use std::{
     fmt::{self, Debug, Formatter},
 };
 
-use futures_util::{future::LocalBoxFuture, Sink, Stream};
+use futures_util::{Sink, Stream};
 
 use crate::{
     common::socket::{BoxedSocketRx, BoxedSocketTx, SocketMessage},
@@ -21,9 +21,6 @@ pub mod http;
 #[cfg(feature = "mock_client")]
 pub mod mock;
 
-/// A type alias that represents a result of a call.
-pub type CallResult<'a, T, Err> = LocalBoxFuture<'a, Result<T, TransportError<Err>>>;
-
 /// Error type that transports need to return.
 pub enum TransportError<Err> {
     /// A transport specific error.
